@@ -54,7 +54,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype AbstractButton
     \inherits Control
-    \instantiates QQuickAbstractButton
+//!     \instantiates QQuickAbstractButton
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-buttons
@@ -105,6 +105,7 @@ QT_BEGIN_NAMESPACE
     \qmlsignal QtQuick.Controls::AbstractButton::pressAndHold()
 
     This signal is emitted when the button is interactively pressed and held down by the user via touch or mouse.
+    It is not emitted when \l autoRepeat is enabled.
 */
 
 /*!
@@ -175,7 +176,7 @@ void QQuickAbstractButtonPrivate::handleRelease(const QPointF &point)
 
     if (wasPressed) {
         emit q->released();
-        if (!wasHeld)
+        if (!wasHeld && !wasDoubleClick)
             trigger();
     } else {
         emit q->canceled();
@@ -185,6 +186,8 @@ void QQuickAbstractButtonPrivate::handleRelease(const QPointF &point)
         stopPressRepeat();
     else
         stopPressAndHold();
+
+    wasDoubleClick = false;
 }
 
 void QQuickAbstractButtonPrivate::handleUngrab()
@@ -198,9 +201,15 @@ void QQuickAbstractButtonPrivate::handleUngrab()
     q->setPressed(false);
     stopPressRepeat();
     stopPressAndHold();
+    wasDoubleClick = false;
     emit q->canceled();
 }
 
+bool QQuickAbstractButtonPrivate::acceptKeyClick(Qt::Key key) const
+{
+    return key == Qt::Key_Space;
+}
+
 bool QQuickAbstractButtonPrivate::isPressAndHoldConnected()
 {
     Q_Q(QQuickAbstractButton);
@@ -449,7 +458,9 @@ QQuickAbstractButton::~QQuickAbstractButton()
     d->removeImplicitSizeListener(d->indicator);
     if (d->group)
         d->group->removeButton(this);
+#if QT_CONFIG(shortcut)
     d->ungrabShortcut();
+#endif
 }
 
 /*!
@@ -651,6 +662,9 @@ void QQuickAbstractButton::setAutoExclusive(bool exclusive)
     This property holds whether the button repeats \l pressed(), \l released()
     and \l clicked() signals while the button is pressed and held down.
 
+    If this property is set to \c true, the \l pressAndHold() signal will not
+    be emitted.
+
     The default value is \c false.
 
     The initial delay and the repetition interval are defined in milliseconds
@@ -729,7 +743,7 @@ void QQuickAbstractButton::setIndicator(QQuickItem *indicator)
 
     \include qquickicon.qdocinc grouped-properties
 
-    \sa text, display, {Icons in Qt Quick Controls 2}
+    \sa text, display, {Icons in Qt Quick Controls}
 */
 
 QQuickIcon QQuickAbstractButton::icon() const
@@ -1029,7 +1043,7 @@ void QQuickAbstractButton::keyPressEvent(QKeyEvent *event)
 {
     Q_D(QQuickAbstractButton);
     QQuickControl::keyPressEvent(event);
-    if (event->key() == Qt::Key_Space) {
+    if (d->acceptKeyClick(static_cast<Qt::Key>(event->key()))) {
         d->setPressPoint(QPoint(qRound(width() / 2), qRound(height() / 2)));
         setPressed(true);
 
@@ -1045,7 +1059,7 @@ void QQuickAbstractButton::keyReleaseEvent(QKeyEvent *event)
 {
     Q_D(QQuickAbstractButton);
     QQuickControl::keyReleaseEvent(event);
-    if (event->key() == Qt::Key_Space) {
+    if (d->acceptKeyClick(static_cast<Qt::Key>(event->key()))) {
         setPressed(false);
 
         nextCheckState();
@@ -1067,8 +1081,10 @@ void QQuickAbstractButton::mousePressEvent(QMouseEvent *event)
 
 void QQuickAbstractButton::mouseDoubleClickEvent(QMouseEvent *event)
 {
+    Q_D(QQuickAbstractButton);
     QQuickControl::mouseDoubleClickEvent(event);
     emit doubleClicked();
+    d->wasDoubleClick = true;
 }
 
 void QQuickAbstractButton::timerEvent(QTimerEvent *event)
@@ -1115,7 +1131,7 @@ void QQuickAbstractButton::buttonChange(ButtonChange change)
         break;
     case ButtonTextChange: {
         const QString txt = text();
-        setAccessibleName(txt);
+        maybeSetAccessibleName(txt);
 #if QT_CONFIG(shortcut)
         setShortcut(QKeySequence::mnemonic(txt));
 #endif
@@ -1141,7 +1157,7 @@ void QQuickAbstractButton::accessibilityActiveChanged(bool active)
 
     Q_D(QQuickAbstractButton);
     if (active) {
-        setAccessibleName(text());
+        maybeSetAccessibleName(text());
         setAccessibleProperty("pressed", d->pressed);
         setAccessibleProperty("checked", d->checked);
         setAccessibleProperty("checkable", d->checkable);
@@ -1150,6 +1166,10 @@ void QQuickAbstractButton::accessibilityActiveChanged(bool active)
 
 QAccessible::Role QQuickAbstractButton::accessibleRole() const
 {
+    Q_D(const QQuickAbstractButton);
+    if (d->checkable) {
+        return QAccessible::CheckBox;
+    }
     return QAccessible::Button;
 }
 #endif
